/* This code will enable the scroll wheel on 
 * Logitech mice with the M-S48 designation which 
 * otherwise will not work under Xwindows
 *
 * This code should be run as root as it requires
 * access to the mouse port.
 *
 * Example Usage:
 *
 * enable_wheel /dev/psaux       (specify your device)
 * 
 * or
 *
 * enable_wheel                  (defaults to /dev/mouse)
 *
 * $Id: enable_wheel.c,v 0.3 1999/07/15 19:35:40 norman Exp norman $
 * $Log: enable_wheel.c,v $
 * Revision 0.3  1999/07/15 19:35:40  norman
 * Fixed typo in revision string of usage message
 *
 * Revision 0.2  1999/07/15 19:32:00  norman
 * Added help flags for command usage.
 *
 * Revision 0.1  1999/07/15 19:10:27  norman
 * Code has been designed around initialization of PS2 style
 * mouse devices with command definitions consistent with
 * the standard linux character device driver (linux/drivers/char/psaux.c)
 *
 * Revision 1.1  1999/07/15 19:08:21  norman
 * Initial revision
 *
 *
 */

/* The following are defines that are made in /usr/src/linux/char/psaux.c regarding
   general control of the PS/2 port */

/* aux device commands */
#define AUX_SET_RES     0xe8            /* set resolution */
#define AUX_SET_SCALE11 0xe6            /* set 1:1 scaling */
#define AUX_SET_SCALE21 0xe7            /* set 2:1 scaling */
#define AUX_GET_SCALE   0xe9            /* get scaling factor */
#define AUX_SET_STREAM  0xea            /* set stream mode */
#define AUX_SET_SAMPLE  0xf3            /* set sample rate */
#define AUX_ENABLE_DEV  0xf4            /* enable aux device */
#define AUX_DISABLE_DEV 0xf5            /* disable aux device */
#define AUX_RESET       0xff            /* reset aux device */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
// #include <termios.h>

int main(int argc,char** argv)
{

  const char disable_PS2      =  AUX_DISABLE_DEV;
  const char enable_PS2       =  AUX_ENABLE_DEV;
  const char reset_PS2        =  AUX_RESET;
  const char enable_logitech[]= {AUX_SET_SCALE11,
				 AUX_SET_RES, 0,
				 AUX_SET_RES, 3,
				 AUX_SET_RES, 2,
				 AUX_SET_RES, 1,
				 AUX_SET_SCALE11,
				 AUX_SET_RES, 3,
				 AUX_SET_RES, 1,
				 AUX_SET_RES, 2,
				 AUX_SET_RES, 3};

  const char enable_wheel[]   = {AUX_SET_SAMPLE,200};  
  int file_descriptor;
  int i;

  /* Check to see if the user requested command help */
  for(i=0;i!=argc;i++)
    {
      /* Accept either -h or --help as valid help flags */
      if( !(strcmp(argv[i],"-h")) || !(strcmp(argv[i],"--help")) )
	{
	  /* Print a usage message */
	  printf(
"
Logitech Wheel Enable $Revision: 0.3 $
By A. Norman (norman@physics.wm.edu)\n
Command Usage:\n
%s [device]\n
Where device should be the character device that the mouse
is active on.  If not specified it will default to /dev/mouse
",argv[0]);
	  exit(1); /* exit out so the user can restart properly */
	}
      if( !(strcmp(argv[i],"-r")) || !(strcmp(argv[i],"--reset")))
	{
	  /* Reset the PS/2 Device */
	  fprintf(stderr,"Reseting PS/2 Device\n");
	  write(file_descriptor, &disable_PS2, 1); /* Diable the PS/2 Mouse          */
	  write(file_descriptor, &reset_PS2,   1); /* Reset  the PS/2 Mouse          */
	  write(file_descriptor, &enable_PS2,  1); /* Enable the PS/2 Mouse          */
	  fprintf(stderr,"Completed Reset\n");
	  exit(2);
	}
    }

  /* Note: we do not try to get a lock because
     we assume that Xwindows or gpm already has 
     opened the device */

  /* Open a file descriptor for the device */
  if(argc>1)
    {  
      file_descriptor=open(argv[1],O_RDWR);
      if(file_descriptor < 0 ) 
	{
	  fprintf(stderr,"Unable to open device: %s\n",argv[1]);
	  exit(-1);
	}
    }
  else
    {
      file_descriptor=open("/dev/mouse",O_RDWR);
      if(file_descriptor < 0 )
	{
	  fprintf(stderr,"Unable to open device: /dev/mouse\n");
	  exit(-1);
	}
    }

  /* Initialize the PS/2 Mouse Device */
  // fprintf(stderr,"Disabling PS/2 Device\n");
  // write(file_descriptor, &disable_PS2,     1                      ); /* Diable the PS/2 Mouse          */
  // write(file_descriptor, &reset_PS2,       1                      ); /* Reset  the PS/2 Mouse          */
  fprintf(stderr,"Sending Base Initialization String\n");
  fflush(stderr);
  write(file_descriptor, &enable_logitech, sizeof(enable_logitech)); /* Send the Initialization String */
  fprintf(stderr,"Sending Wheel Initialization String\n");
  fflush(stderr);
  write(file_descriptor, &enable_wheel,    sizeof(enable_wheel)   ); /* Send the Wheel Initialization  */
  // fprintf(stderr,"Enabling PS/2 Device\n");
  // write(file_descriptor, &enable_PS2,      1                      ); /* Enable the PS/2 Mouse          */

  /* Close the port */

  fprintf(stderr,"Initialization complete\nWiggle the mouse if the program has not terminated yet\n");
  fflush(stderr);

  close(file_descriptor);

  /* Return to OS */
  return(0);
}
